@@ -27,7 +27,6 @@
 #include <osgEarth/LandCover>
 #include <osgEarth/Metrics>
 #include <osgEarth/Color>
-#include <osgEarth/Math>
 
 #include <osgDB/FileNameUtils>
 #include <osgDB/FileUtils>
@@ -47,7 +46,6 @@
 #include <gdal_vrt.h>
 #include <ogr_spatialref.h>
 #include <cpl_string.h>
-#include <gdal_alg_priv.h>
 
 using namespace osgEarth;
 using namespace osgEarth::GDAL;
@@ -332,28 +330,6 @@ namespace osgEarth
                 *f++ = static_cast<T>(value);
             }
         }
-        
-        void applyScaleAndOffset(GDALRasterBand* band, void* pData, GDALDataType eBufType, int nBufXSize, int nBufYSize)
-        {
-            double scale = band->GetScale();
-            double offset = band->GetOffset();
-
-            if (scale != 1.0 || offset != 0.0)
-            {
-                int count = nBufXSize * nBufYSize;
-
-                if (eBufType == GDT_Float32)
-                    applyScaleAndOffset<float>(pData, count, scale, offset);
-                else if (eBufType == GDT_Float64)
-                    applyScaleAndOffset<double>(pData, count, scale, offset);
-                else if (eBufType == GDT_Int16)
-                    applyScaleAndOffset<short>(pData, count, scale, offset);
-                else if (eBufType == GDT_Int32)
-                    applyScaleAndOffset<int>(pData, count, scale, offset);
-                else if (eBufType == GDT_Byte)
-                    applyScaleAndOffset<char>(pData, count, scale, offset);
-            }
-        }
 
         // GDALRasterBand::RasterIO helper method
         bool rasterIO(
@@ -411,7 +387,24 @@ namespace osgEarth
             }
             else
             {
-                applyScaleAndOffset(band, pData, eBufType, nBufXSize, nBufYSize);
+                double scale = band->GetScale();
+                double offset = band->GetOffset();
+
+                if (scale != 1.0 || offset != 0.0)
+                {
+                    int count = nBufXSize * nBufYSize;
+
+                    if (eBufType == GDT_Float32)
+                        applyScaleAndOffset<float>(pData, count, scale, offset);
+                    else if (eBufType == GDT_Float64)
+                        applyScaleAndOffset<double>(pData, count, scale, offset);
+                    else if (eBufType == GDT_Int16)
+                        applyScaleAndOffset<short>(pData, count, scale, offset);
+                    else if (eBufType == GDT_Int32)
+                        applyScaleAndOffset<int>(pData, count, scale, offset);
+                    else if (eBufType == GDT_Byte)
+                        applyScaleAndOffset<char>(pData, count, scale, offset);
+                }
             }
 
             return (err == CE_None);
@@ -677,10 +670,6 @@ GDAL::Driver::open(
     pixelToGeo(0.0, _warpedDS->GetRasterYSize(), minX, minY);
     pixelToGeo(_warpedDS->GetRasterXSize(), 0.0, maxX, maxY);
 
-    // record the AREA_OR_POINT metadata if available (default to AREA)
-    auto* pora = _warpedDS->GetMetadataItem("AREA_OR_POINT");
-    _pixelIsArea = (pora == nullptr) || toLower(std::string(pora)) == "area";
-
     //OE_INFO << LC << INDENT << gdalOptions().url()->full() << ": pixelToGeo: " << minX << ", " << minY << " -> " << maxX << ", " << maxY << std::endl;
 
     // If we don't have a profile yet, that means this is a projected dataset
@@ -708,7 +697,7 @@ GDAL::Driver::open(
     double maxResolution = osg::minimum(resolutionX, resolutionY);
 
     if (verbose)
-        OE_INFO << LC << source << ": resolution= " << resolutionX << "x" << resolutionY << " max=" << maxResolution << std::endl;
+        OE_DEBUG << LC << source << ": resolution= " << resolutionX << "x" << resolutionY << " max=" << maxResolution << std::endl;
 
     if (_maxDataLevel.isSet())
     {
@@ -728,7 +717,7 @@ GDAL::Driver::open(
         }
 
         if (verbose)
-            OE_INFO << LC << source << ": max data level= " << _maxDataLevel.get() << std::endl;
+            OE_DEBUG << LC << source << ": max data level= " << _maxDataLevel.get() << std::endl;
     }
 
     // If the input dataset is a VRT, then get the individual files in the dataset and use THEM for the DataExtents.
@@ -741,10 +730,13 @@ GDAL::Driver::open(
     // record the data extent in profile space:
     _bounds = Bounds(minX, minY, 0.0, maxX, maxY, 0.0);
 
+    const char* pora = _srcDS->GetMetadataItem("AREA_OR_POINT");
+    bool is_area = pora != nullptr && Strings::toLower(std::string(pora)) == "area";
+
     bool clamped = false;
     if (srs->isGeographic())
     {
-        if (_pixelIsArea && (_bounds.xMin() < -180.0 || _bounds.xMax() > 180.0))
+        if (is_area && (_bounds.xMin() < -180.0 || _bounds.xMax() > 180.0))
         {
             _bounds.xMin() += resolutionX * 0.5;
             _bounds.xMax() -= resolutionX * 0.5;
@@ -757,7 +749,7 @@ GDAL::Driver::open(
             clamped = true;
         }
 
-        if (_pixelIsArea && (_bounds.yMin() < -90.0 || _bounds.yMax() > 90.0))
+        if (is_area && (_bounds.yMin() < -90.0 || _bounds.yMax() > 90.0))
         {
             _bounds.yMin() += resolutionY * 0.5;
             _bounds.yMax() -= resolutionY * 0.5;
@@ -830,28 +822,7 @@ GDAL::Driver::geoToPixel(double geoX, double geoY, double &x, double &y)
 }
 
 bool
-GDAL::Driver::isValidValue(float v, float noDataValue) const
-{
-    //Check to see if the value is equal to the bands specified no data
-    if (noDataValue == v)
-        return false;
-
-    //Check to see if the value is equal to the user specified nodata value
-    if (_noDataValue.isSetTo(v))
-        return false;
-
-    //Check to see if the user specified a custom min/max
-    if (_minValidValue.isSet() && v < _minValidValue.get())
-        return false;
-
-    if (_maxValidValue.isSet() && v > _maxValidValue.get())
-        return false;
-
-    return true;
-}
-
-bool
-GDAL::Driver::isValidValue(float v, GDALRasterBand* band) const
+GDAL::Driver::isValidValue(float v, GDALRasterBand* band)
 {
     float bandNoData = -32767.0f;
     int success;
@@ -880,122 +851,11 @@ GDAL::Driver::isValidValue(float v, GDALRasterBand* band) const
 }
 
 float
-GDAL::Driver::getInterpolatedDEMValueWorkspace(GDALRasterBand* band, double u, double v, float* data, int width, int height)
-{
-    float result = 0.0f;
-
-    // extract the no-data value:
-    int success;
-    float value = band->GetNoDataValue(&success);
-    float noDataValue = success ? value : -32767.0f;
-
-    // clamp our sampling unit coordinates to the valid range
-    // note: (u,v) progresses from north/+y at the top to south/-y at the bottom.
-    double c = clamp(u, 0.0, 1.0) * (double)(width);
-    double r = clamp(v, 0.0, 1.0) * (double)(height);
-
-    if (gdalOptions().interpolation() == INTERP_NEAREST)
-    {
-        int x = clamp(c, 0.0, (double)width - 1);
-        int y = clamp(r, 0.0, (double)height - 1);
-
-        result = data[y * width + x];
-
-        if (!isValidValue(result, noDataValue))
-        {
-            result = NO_DATA_VALUE;
-        }
-    }
-    else
-    {
-        // Get the four nearest pixels:
-        int col_min, col_max, row_min, row_max;
-
-#if 0
-        col_min = (fract(c) < 0.5) ? (int)c - 1 : (int)c;
-        col_max = clamp(col_min + 1, 0, width - 1);
-        col_min = clamp(col_min, 0, width - 1);
-
-        row_min = (fract(r) < 0.5) ? (int)r - 1 : (int)r;
-        row_max = clamp(row_min + 1, 0, height - 1);
-        row_min = clamp(row_min, 0, height - 1);
-#else   
-        col_min = clamp((int)floor(c), 0, width - 1);
-        col_max = clamp((int)ceil(c), 0, width - 1);
-        row_min = clamp((int)floor(r), 0, height - 1);
-        row_max = clamp((int)ceil(r), 0, height - 1);
-#endif
-
-        // we will use NSEW here for clarity even though some projections are not NSEW aligned.
-        // North means +y, South means -y, East means +x, West means -x
-        auto NW = data[row_min * width + col_min];
-        auto NE = data[row_min * width + col_max];
-        auto SW = data[row_max * width + col_min];
-        auto SE = data[row_max * width + col_max];
-
-        if ((!isValidValue(NW, noDataValue)) || (!isValidValue(NE, noDataValue)) || (!isValidValue(SW, noDataValue)) || (!isValidValue(SE, noDataValue)))
-        {
-            result = NO_DATA_VALUE;
-        }
-        else
-        {
-            double west_weight = clamp(((double)col_max + 0.5) - c, 0.0, 1.0);
-            double south_weight = clamp(((double)row_max + 0.5) - r, 0.0, 1.0);
-
-            if (gdalOptions().interpolation() == INTERP_AVERAGE)
-            {
-                double h0 = west_weight * south_weight * (double)SW;
-                double h1 = west_weight * (1.0 - south_weight) * (double)NW;
-                double h2 = (1.0 - west_weight) * south_weight * (double)SE;
-                double h3 = (1.0 - west_weight) * (1.0 - south_weight) * (double)NE;
-                result = (float)(h0 + h1 + h2 + h3);
-            }
-            else //if (gdalOptions().interpolation() == INTERP_BILINEAR)
-            {
-                double south = west_weight * (double)SW + (1.0 - west_weight) * (double)SE;
-                double north = west_weight * (double)NW + (1.0 - west_weight) * (double)NE;
-                result = south_weight * south + (1.0 - south_weight) * north;
-            }
-        }
-    }
-
-    return result;
-}
-
-float
-GDAL::Driver::getInterpolatedDEMValue(GDALRasterBand* band, double x, double y, bool applyOffset)
+GDAL::Driver::getInterpolatedValue(GDALRasterBand* band, double x, double y, bool applyOffset)
 {
     double r, c;
     geoToPixel(x, y, c, r);
 
-#if GDAL_VERSION_NUM >= 3100000 // 3.10+
-    GDALRIOResampleAlg alg = GRIORA_NearestNeighbour;
-
-    switch (gdalOptions().interpolation().value())
-    {
-    case INTERP_AVERAGE:
-        alg = GRIORA_Average;
-        break;
-    case INTERP_BILINEAR:
-        alg = GRIORA_Bilinear;
-        break;
-    case INTERP_CUBIC:
-        alg = GRIORA_Cubic;
-        break;
-    case INTERP_CUBICSPLINE:
-        alg = GRIORA_CubicSpline;
-        break;
-    }
-
-    // this function applies the 1/2 pixel offset for us for DEMs
-    double realPart = 0.0;
-    auto err = band->InterpolateAtPoint(c, r, alg, &realPart, nullptr);
-    if (err == CE_None)
-    {
-        return (float)realPart;
-    }
-#endif
-
     if (applyOffset)
     {
         //Apply half pixel offset
@@ -1030,7 +890,6 @@ GDAL::Driver::getInterpolatedDEMValue(GDALRasterBand* band, double x, double y,
     if (c < 0 || r < 0 || c > _warpedDS->GetRasterXSize() - 1 || r > _warpedDS->GetRasterYSize() - 1)
         return NO_DATA_VALUE;
 
-
     if (gdalOptions().interpolation() == INTERP_NEAREST)
     {
         rasterIO(band, GF_Read, osg::round(c), osg::round(r), 1, 1, &result, 1, 1, GDT_Float32, 0, 0);
@@ -1526,16 +1385,10 @@ GDAL::Driver::createImage(const TileKey& key,
     return image.release();
 }
 
-namespace
-{
-    // per-thread raster sampling workspace for createHeightField
-    // to avoid heap allocations
-    thread_local std::vector<float> workspace;
-}
-
-#if 0
 osg::HeightField*
-GDAL::Driver::createHeightField(const TileKey& key, unsigned tileSize, ProgressCallback* progress)
+GDAL::Driver::createHeightField(const TileKey& key,
+    unsigned tileSize,
+    ProgressCallback* progress)
 {
     if (_maxDataLevel.isSet() && key.getLevelOfDetail() > _maxDataLevel.get())
     {
@@ -1543,87 +1396,83 @@ GDAL::Driver::createHeightField(const TileKey& key, unsigned tileSize, ProgressC
         return NULL;
     }
 
-    // Allocate the heightfield
+    //GDAL_SCOPED_LOCK;
+
+    //Allocate the heightfield
     osg::ref_ptr<osg::HeightField> hf = new osg::HeightField;
     hf->allocate(tileSize, tileSize);
 
     if (intersects(key))
-    {        
-        // Extract the extents of the tile
-        double tile_xmin, tile_ymin, tile_xmax, tile_ymax;
-        key.getExtent().getBounds(tile_xmin, tile_ymin, tile_xmax, tile_ymax);
+    {
+        //Get the meter extents of the tile
+        double xmin, ymin, xmax, ymax;
+        key.getExtent().getBounds(xmin, ymin, xmax, ymax);
 
-        // Assume the first band contains our data
-        auto* band = _warpedDS->GetRasterBand(1);
-
-        // Calculate the pixel extents of the tile:
-        double tile_col_min, tile_col_max;
-        double tile_row_min, tile_row_max;
-        geoToPixel(tile_xmin, tile_ymin, tile_col_min, tile_row_max);
-        geoToPixel(tile_xmax, tile_ymax, tile_col_max, tile_row_min);
-
-        tile_col_min -= 0.5;
-        tile_col_max -= 0.5;
-        tile_row_min -= 0.5;
-        tile_row_max -= 0.5;
-
-        double col_min = clamp(tile_col_min, 0.0, (double)band->GetXSize() - 1.0);
-        double col_max = clamp(tile_col_max, 0.0, (double)band->GetXSize() - 1.0);
-        double row_min = clamp(tile_row_min, 0.0, (double)band->GetYSize() - 1.0);
-        double row_max = clamp(tile_row_max, 0.0, (double)band->GetYSize() - 1.0);
-
-        // Allocate a read workspace for RasterIO.
-        int workspace_width = tileSize, workspace_height = tileSize;
-
-        // note: workspace is a thread_local vector, see above
-        workspace.assign(workspace_width * workspace_height, NO_DATA_VALUE);
-
-        GDALRIOResampleAlg resample = GRIORA_NearestNeighbour;
-        switch (gdalOptions().interpolation().value())
+        // Try to find a FLOAT band
+        GDALRasterBand* band = findBandByDataType(_warpedDS, GDT_Float32);
+        if (band == NULL)
         {
-        case INTERP_AVERAGE:
-            //psExtraArg.eResampleAlg = GRIORA_Average;
-            // for some reason gdal's average resampling produces artifacts occasionally for imagery at higher levels.
-            // for now we'll just use bilinear interpolation under the hood until we can understand what is going on.
-            resample = GRIORA_Bilinear;
-            break;
-        case INTERP_BILINEAR:
-            resample = GRIORA_Bilinear;
-            break;
-        case INTERP_CUBIC:
-            resample = GRIORA_Cubic;
-            break;
-        case INTERP_CUBICSPLINE:
-            resample = GRIORA_CubicSpline;
-            break;
+            // Just get first band
+            band = _warpedDS->GetRasterBand(1);
         }
 
-        auto read_err = band->ReadRaster(
-            workspace,
-            col_min, row_min,
-            col_max - col_min + 1, row_max - row_min + 1,
-            workspace_width, workspace_height,
-            GRIORA_NearestNeighbour);
-
-        if (read_err != CE_None)
+        if (gdalOptions().interpolation() == INTERP_NEAREST)
         {
-            //OE_WARN << LC << "RasterIO failed.\n";
-            return nullptr;
-        }
+            double colMin, colMax;
+            double rowMin, rowMax;
+            geoToPixel(xmin, ymin, colMin, rowMax);
+            geoToPixel(xmax, ymax, colMax, rowMin);
+            std::vector<float> buffer(tileSize * tileSize, NO_DATA_VALUE);
 
-        // Fill the heightfield by transforming the tile's coordinates to the buffer's coordinates
-        // and sampling the buffer.
-        for (unsigned r = 0; r < tileSize; ++r)
-        {
-            int inv_r = workspace_height - r - 1;
-            for (unsigned c = 0; c < tileSize; ++c)
+            int iColMin = floor(colMin);
+            int iColMax = ceil(colMax);
+            int iRowMin = floor(rowMin);
+            int iRowMax = ceil(rowMax);
+            int iNumCols = iColMax - iColMin + 1;
+            int iNumRows = iRowMax - iRowMin + 1;
+
+            int iWinColMin = osg::maximum(0, iColMin);
+            int iWinColMax = osg::minimum(_warpedDS->GetRasterXSize() - 1, iColMax);
+            int iWinRowMin = osg::maximum(0, iRowMin);
+            int iWinRowMax = osg::minimum(_warpedDS->GetRasterYSize() - 1, iRowMax);
+            int iNumWinCols = iWinColMax - iWinColMin + 1;
+            int iNumWinRows = iWinRowMax - iWinRowMin + 1;
+
+            int iBufColMin = osg::round((iWinColMin - iColMin) / double(iNumCols - 1) * (tileSize - 1));
+            int iBufColMax = osg::round((iWinColMax - iColMin) / double(iNumCols - 1) * (tileSize - 1));
+            int iBufRowMin = osg::round((iWinRowMin - iRowMin) / double(iNumRows - 1) * (tileSize - 1));
+            int iBufRowMax = osg::round((iWinRowMax - iRowMin) / double(iNumRows - 1) * (tileSize - 1));
+            int iNumBufCols = iBufColMax - iBufColMin + 1;
+            int iNumBufRows = iBufRowMax - iBufRowMin + 1;
+
+            int startOffset = iBufRowMin * tileSize + iBufColMin;
+            int lineSpace = tileSize * sizeof(float);
+
+            rasterIO(band, GF_Read, iWinColMin, iWinRowMin, iNumWinCols, iNumWinRows, &buffer[startOffset], iNumBufCols, iNumBufRows, GDT_Float32, 0, lineSpace);
+
+            for (unsigned r = 0, ir = tileSize - 1; r < tileSize; ++r, --ir)
             {
-                hf->setHeight(c, r, workspace[c + inv_r * workspace_width]);
+                for (unsigned c = 0; c < tileSize; ++c)
+                {
+                    hf->setHeight(c, ir, _linearUnits * buffer[r * tileSize + c]);
+                }
+            }
+        }
+        else
+        {
+            double dx = (xmax - xmin) / (tileSize - 1);
+            double dy = (ymax - ymin) / (tileSize - 1);
+            for (unsigned r = 0; r < tileSize; ++r)
+            {
+                double geoY = ymin + (dy * (double)r);
+                for (unsigned c = 0; c < tileSize; ++c)
+                {
+                    double geoX = xmin + (dx * (double)c);
+                    float h = getInterpolatedValue(band, geoX, geoY) * _linearUnits;
+                    hf->setHeight(c, r, h);
+                }
             }
         }
-
-        // Apply any scale/offset found in the source:
-        applyScaleAndOffset(band, (void*)hf->getFloatArray()->getDataPointer(), GDT_Float32, tileSize, tileSize);
     }
     else
     {
@@ -1632,127 +1481,6 @@ GDAL::Driver::createHeightField(const TileKey& key, unsigned tileSize, ProgressC
     }
     return hf.release();
 }
-#else
-osg::HeightField*
-GDAL::Driver::createHeightField(const TileKey& key, unsigned tileSize, ProgressCallback* progress)
-{
-    if (_maxDataLevel.isSet() && key.getLevelOfDetail() > _maxDataLevel.get())
-    {
-        //OE_NOTICE << "Reached maximum data resolution key=" << key.getLevelOfDetail() << " max=" << _maxDataLevel <<  std::endl;
-        return NULL;
-    }
-
-    // Allocate the heightfield
-    osg::ref_ptr<osg::HeightField> hf = new osg::HeightField;
-    hf->allocate(tileSize, tileSize);
-
-    if (intersects(key))
-    {
-        // Extract the extents of the tile
-        double tile_xmin, tile_ymin, tile_xmax, tile_ymax;
-        key.getExtent().getBounds(tile_xmin, tile_ymin, tile_xmax, tile_ymax);
-
-        // Sampling intervals:
-        double dx = (tile_xmax - tile_xmin) / (tileSize - 1);
-        double dy = (tile_ymax - tile_ymin) / (tileSize - 1);
-
-        // Assume the first band contains our data
-        auto* band = _warpedDS->GetRasterBand(1);
-
-        if (_pixelIsArea && gdalOptions().interpolation() != INTERP_NEAREST)
-        {
-            // Note. This method always works, but it's slow.
-            // It wound be ideal to use the method in the "else" block but it
-            // does not yet work with the half-pixel shift that is required for DEMs.
-            for (unsigned r = 0; r < tileSize; ++r)
-            {
-                double y = tile_ymin + (dy * (double)r);
-                for (unsigned c = 0; c < tileSize; ++c)
-                {
-                    double x = tile_xmin + (dx * (double)c);
-                    float h = getInterpolatedDEMValue(band, x, y, true) * _linearUnits;
-                    hf->setHeight(c, r, h);
-                }
-            }
-        }
-        
-        else // _pixelisPoint && gdalOptions().interpolation() == INTERP_NEAREST
-        {
-            // Calculate the pixel extents of the tile:
-            double tile_col_min, tile_col_max;
-            double tile_row_min, tile_row_max;
-            geoToPixel(tile_xmin, tile_ymin, tile_col_min, tile_row_max);
-            geoToPixel(tile_xmax, tile_ymax, tile_col_max, tile_row_min);
-
-            const double ws_buffer = 0.5; // greater values cause gaps. I don't know why!!
-
-            int col_min = std::max(0.0, floor(tile_col_min - ws_buffer));
-            int col_max = std::min(ceil(tile_col_max + ws_buffer), (double)band->GetXSize() - 1.0);
-            int row_min = std::max(0.0, floor(tile_row_min - ws_buffer));
-            int row_max = std::min(ceil(tile_row_max + ws_buffer), (double)band->GetYSize() - 1.0);
-
-            // Allocate a read workspace for RasterIO.
-            // note: workspace is a thread_local vector, see above
-            int workspace_width = tileSize, workspace_height = tileSize;
-
-            workspace.assign(workspace_width * workspace_height, NO_DATA_VALUE);
-
-            // Read the data, filling the workspace vector from north to south:
-            auto read_error = band->RasterIO(GF_Read,
-                (int)col_min, (int)row_min,
-                (int)col_max - (int)col_min + 1, (int)row_max - (int)row_min + 1,
-                &workspace[0], workspace_width, workspace_height,
-                GDT_Float32, 0, 0);
-
-            if (read_error != CE_None)
-            {
-                //OE_WARN << LC << "RasterIO failed.\n";
-                return nullptr;
-            }
-
-            // Calculate the actual extents of the pixel data in buffer, which will be slightly
-            // different from the tile extents due to the buffering.
-            // Remember to flip the Y axis.
-            // The "+1"s expand the maximums to include the full extent of the last pixel
-            double buf_xmin, buf_ymin, buf_xmax, buf_ymax;
-            pixelToGeo(col_min, row_max + 1, buf_xmin, buf_ymin);
-            pixelToGeo(col_max + 1, row_min, buf_xmax, buf_ymax);
-
-            // Just to deal with floating point precision issues
-            const double epsilon = 1e-6;
-
-            // Fill the heightfield by transforming the tile's coordinates to the buffer's coordinates
-            // and sampling the buffer.
-            for (unsigned r = 0; r < tileSize; ++r)
-            {
-                double y = tile_ymin + (dy * (double)r);
-                double v = (y - buf_ymin) / (buf_ymax - buf_ymin);
-                if (equivalent(v, 0.0, epsilon)) v = 0.0;
-
-                for (unsigned c = 0; c < tileSize; ++c)
-                {
-                    double x = tile_xmin + (dx * (double)c);
-                    double u = (x - buf_xmin) / (buf_xmax - buf_xmin);
-                    if (equivalent(u, 0.0, epsilon)) u = 0;
-
-                    // invert v since the buffer from rasterio is top-down:
-                    float h = getInterpolatedDEMValueWorkspace(band, u, 1.0 - v, &workspace[0], workspace_width, workspace_height) * _linearUnits;
-                    hf->setHeight(c, r, h);
-                }
-            }
-        }
-
-        // Apply any scale/offset found in the source:
-        applyScaleAndOffset(band, (void*)hf->getFloatArray()->getDataPointer(), GDT_Float32, tileSize, tileSize);
-    }
-    else
-    {
-        std::vector<float>& heightList = hf->getHeightList();
-        std::fill(heightList.begin(), heightList.end(), NO_DATA_VALUE);
-    }
-    return hf.release();
-}
-#endif
 
 osg::HeightField*
 GDAL::Driver::createHeightFieldWithVRT(const TileKey& key,
