@@ -355,10 +355,7 @@ GeoPoint::transformResolution(const Distance& resolution, const UnitsType& outUn
             refLatDegrees);
     }
 
-    double d = getSRS()->transformDistance(resolution, outUnits, refLatDegrees);
-
-    //double d = resolution.asDistance(outUnits, refLatDegrees);
-
+    double d = resolution.asDistance(outUnits, refLatDegrees);
     return Distance(d, outUnits);
 }
 
@@ -1660,9 +1657,6 @@ GeoExtent::createWorldBoundingSphere(double minElev, double maxElev) const
 {
     osg::BoundingSphered bs;
 
-    if (!getSRS())
-        return bs;
-
     if (getSRS()->isProjected())
     {
         osg::Vec3d w;
